;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x9a9696bdee8675bdbe6e7b09325ef7cf . "./code.cg2js/cg-acache.fasl")
(#xcc33e0c745d5a3ac3b30ff12c4d17625 . "./code.cg2js/cg-base.fasl")
(#xd4cedb607428a257ac802d6b61d71c2f . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x7c44f66c9e671d7c21eb671bb999ad15 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xe23e9864b109f0249dc1e8d77db9beeb . "./code.cg2js/cg-bitmap-stream.fasl")
(#x1224735e4faa5b82dc4efcb5fa348072 . "./code.cg2js/cg-button.fasl")
(#x5fbd1954f8b26c89cb5fcbccc6dcff1b . "./code.cg2js/cg-calendar.fasl")
(#xaa332b06cf45e9e7a390cb92a77c2b9f . "./code.cg2js/cg-caret.fasl")
(#x3c411e9a7a69311078afa2e9f8abc0dc . "./code.cg2js/cg-chart-or-plot.fasl")
(#x65d773f07fbcf5334a3c28f480e46152 . "./code.cg2js/cg-chart-widget.fasl")
(#x4dd9cf44e8845346ea59f3a18b417d50 . "./code.cg2js/cg-check-box.fasl")
(#x895633ebe604ef757341a616e71678f0 . "./code.cg2js/cg-choice-list.fasl")
(#xc543550f033de50e160bc7786d94252c . "./code.cg2js/cg-choose-printer.fasl")
(#x66b746c67884e09c6f37fc3d11f9229e . "./code.cg2js/cg-class-grid.fasl")
(#x01fee1fa456fb2447c9fba4c9c31cb23 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x954eb24602fa16eb8eb5f600a1e29749 . "./code.cg2js/cg-class-support.fasl")
(#x17e3774c6efecc0e993c983765c39b74 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x049beab052f195f4383d3773f023bcc5 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x3ca51d9851ee7046fb30a864afc8dceb . "./code.cg2js/cg-clipboard.fasl")
(#xa7a097974a9a7577befff2165ec9e13c . "./code.cg2js/cg-color-dialog.fasl")
(#xac3034ccd04b31aaae472283fb8dc3a7 . "./code.cg2js/cg-combo-box.fasl")
(#xbd354b06ca3011ff1ac27a03c95d10d9 . "./code.cg2js/cg-common-control.fasl")
(#x8a0dc9eb99c212c4c6be655139bbce64 . "./code.cg2js/cg-comtab.fasl")
(#x5ecbd95aa8ad75e52917ce182ec551c9 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x13d7e5f2df2904d70de18321604aef8f . "./code.cg2js/cg-curve.fasl")
(#xc75a71b74ec8d6f45bbd033064981814 . "./code.cg2js/cg-dde-utils.fasl")
(#xe3138f4d589ef1bfe81263beb2a05541 . "./code.cg2js/cg-dev.fasl")
(#x07a624042cc9d235677d6531e1212ff0 . "./code.cg2js/cg-dialog-item.fasl")
(#x3ccd263adddee288577d4fc2ef3871fa . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xa2df8fd239a15cf333dad87331f12788 . "./code.cg2js/cg-directory-dialog.fasl")
(#x5487d29a72b77009dc09fdc070aeb8cc . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xe3aaa3973513ef56269c6205d9b97fc6 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x2d051289b6368861ed25ff2efb4487f3 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x9c3cc1d51c65c9097905eb94865d1c9f . "./code.cg2js/cg-drawable.fasl")
(#x3736f1728abc5b20defb65c9bec1c459 . "./code.cg2js/cg-dropping-outline.fasl")
(#x549a424a5e40048c4e31f1e329ff2d34 . "./code.cg2js/cg-edit-in-place.fasl")
(#xdb2b53d9d806c60211c668e5468f8c59 . "./code.cg2js/cg-editable-text.fasl")
(#x5fade54ece859a6e4b11a974407d1965 . "./code.cg2js/cg-file-dialog.fasl")
(#x8d4afe4cbd8da83b8bd66e9cb9296df5 . "./code.cg2js/cg-fill-texture.fasl")
(#x71ea31e429304a172c11b5f6e32ec1e2 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x94f4af0db6bab5d37c519160ba892274 . "./code.cg2js/cg-font-dialog.fasl")
(#x77703e0f1a4607a99b7924fe3a39bbe5 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x9c3c468fb00076348d4291fba312efa1 . "./code.cg2js/cg-get-pixmap.fasl")
(#x637a3a08891c3c2131cde0dca15137ff . "./code.cg2js/cg-get-position.fasl")
(#x9549112e7172b5ec444880d316159c50 . "./code.cg2js/cg-graphics-context.fasl")
(#x2cb6e69ac764493c0b1ec8c76f01fc72 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xfd909b227e5455e10c0a7d64e107d970 . "./code.cg2js/cg-grid-widget.fasl")
(#xb915db042d3aa42b33a5e37dc7934bb5 . "./code.cg2js/cg-group-box.fasl")
(#x781c93ae403abb1466d7d074a41fedef . "./code.cg2js/cg-header-control.fasl")
(#xc217e5798ef67a063fd8a5237e317f68 . "./code.cg2js/cg-hotspot.fasl")
(#x7b45c8fb5887f87b6e94c77a4546baa3 . "./code.cg2js/cg-html-dialog.fasl")
(#xa58d2f39c532010e73d1092edaf77b37 . "./code.cg2js/cg-html-widget.fasl")
(#x9378d457c7d23d39ee80c424b7e5241f . "./code.cg2js/cg-icon-pixmap.fasl")
(#xe7ac4b7ab21a7909e9116d22d110693b . "./code.cg2js/cg-icon.fasl")
(#x5734b2a1deac516e5032f820727e5ddf . "./code.cg2js/cg-ie.fasl")
(#x639d7f3757f1d83df2e51bac872f7a3d . "./code.cg2js/cg-intersect-posbox.fasl")
(#x59fd5b1b65f56066fe6e46b960491478 . "./code.cg2js/cg-intersect.fasl")
(#xdc2dd4a24f08a828de7e1973dae119fa . "./code.cg2js/cg-item-list.fasl")
(#x01776f0e3a852b8a38c7283e283e52b9 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x13c0eb8bedc283fc4a9f22cd37579193 . "./code.cg2js/cg-lamp.fasl")
(#x3f4ac761470d5441f632d0ad95f33223 . "./code.cg2js/cg-layout.fasl")
(#x7a1bd7ea9e24b7cecdb263c135abe64f . "./code.cg2js/cg-lettered-menu.fasl")
(#xd9bfdd4c6f8180be1474bf8162fbd051 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xa381d3160d9fda5f041bf867e57d82b5 . "./code.cg2js/cg-lisp-text.fasl")
(#x6c3c34a9e76f34e0ade752d35e097fb5 . "./code.cg2js/cg-lisp-widget.fasl")
(#x0c17ba9d4903311e640380956bc69ced . "./code.cg2js/cg-list-view.fasl")
(#x9f0ec6c8e6ec34bcb6ec428ed0f6285d . "./code.cg2js/cg-mci.fasl")
(#xb78112ffd3f28b92a3bbe73bf7aa5544 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xe8cea3941a88287fa68ac29cb753bd7a . "./code.cg2js/cg-menu.fasl")
(#xb5f5ccce36c2b5c8258f38666afe57e0 . "./code.cg2js/cg-message-dialog.fasl")
(#xe3ff7aac628aff07936d02bea44b44af . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xeb442db6675d2ace75bcd9f6af1ff0bc . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x5a452126be7294a0cff7a40fd33d52b1 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x0b15ec0c42090d7766b48c4a9bc170d0 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x47c3da5074e132870cb8ae7b0ff649bb . "./code.cg2js/cg-multi-picture-button.fasl")
(#x8eb37ba59ae6ffc2159135f739be9426 . "./code.cg2js/cg-nodes.fasl")
(#x238a4bd9652bff999728b45436a5a30f . "./code.cg2js/cg-object-editor-layout.fasl")
(#x6dbd6e197113d7819217a795990ca615 . "./code.cg2js/cg-object-editor.fasl")
(#x17f2b9752c44aa382d21e80276a35401 . "./code.cg2js/cg-ocx.fasl")
(#xb285cb6874fa86c1711093b433b3cb1c . "./code.cg2js/cg-os-widget.fasl")
(#xf801af2407707b55f7a757d9b582a2c4 . "./code.cg2js/cg-os-window.fasl")
(#xd549b9de76731c2d48b42adf1beda1d8 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x9244ff35a435124cb0643a3145e9bd99 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xda88c5f0b6b87daabb9ee686206e2dd6 . "./code.cg2js/cg-outline.fasl")
(#x3a65341356c2f81edbe2f0875f69664c . "./code.cg2js/cg-palette.fasl")
(#xc7480e0f98eee8ba76ccf5f626ca9367 . "./code.cg2js/cg-paren-matching.fasl")
(#xa770921993aef708be8d0663c5dbd53d . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x1774edcae8f0b74ab5131e8673e5552b . "./code.cg2js/cg-picture-widget.fasl")
(#x161e492b553c689b925351cc4b54f6ef . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xe4f46a403d1f2d955cfff3418d15b6d9 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x889589d2790c6378d32c7626e7651065 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x0a87dc6b09d086fcbbce089e8242462f . "./code.cg2js/cg-pixmap-widget.fasl")
(#xda046bc27e691b25fe436d6fe63067fe . "./code.cg2js/cg-pixmap.fasl")
(#xeaec35e2e03f0a14a5aa4e1063b004a7 . "./code.cg2js/cg-plot-widget.fasl")
(#xdeafbb3479a84ab76fb83b390416c1f2 . "./code.cg2js/cg-printing.fasl")
(#x2982a67524c90386ee9745ba34dbd557 . "./code.cg2js/cg-progress-indicator.fasl")
(#x1bdd2415d38f5f192c8ddb7be2c2b504 . "./code.cg2js/cg-project-window.fasl")
(#x7236212347bb95ddb867b24a840b4f1a . "./code.cg2js/cg-property.fasl")
(#xea649be67ea72f3ab2b008efc3e4cd71 . "./code.cg2js/cg-radio-button.fasl")
(#x445a568d8da2e2bde1895a0476c63303 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xb63e9cbff52764dab3b8a359f904dd72 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x89bf3ada881453440d3d9f602c15a48c . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x95d2c3cf2f39a94d6f8ff68a7d02d932 . "./code.cg2js/cg-rich-edit.fasl")
(#x5313d1f50347bff9df492926d3d3910b . "./code.cg2js/cg-sample-file-menu.fasl")
(#x5b75aaa6a5e77f3b367c042405f82a0e . "./code.cg2js/cg-scaling-stream.fasl")
(#xf67d9bb2e06f7ce76442c6258d67ebda . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x0ebe536cbc8f8d657d69f328b9052a09 . "./code.cg2js/cg-scroll-bar.fasl")
(#xee28deb10a2dd939b675a502266cee49 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xdbbc1b4498c6995f03655c3f84fef31d . "./code.cg2js/cg-selected-object.fasl")
(#xf054fe21c1f65b6003ef9450224ca989 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xcc228e6946e8d0808c61dfe709a56d65 . "./code.cg2js/cg-split-bar.fasl")
(#x71f9e406f11042ba21f2f71d1ca5ee25 . "./code.cg2js/cg-static-text.fasl")
(#xc465133e76691b50dfec54e12881a5fb . "./code.cg2js/cg-status-bar.fasl")
(#xe01903aa9b5b7e9f930e5a10a73c543f . "./code.cg2js/cg-string-dialog.fasl")
(#x5d80e47625794a93ee2fc51df3d181f2 . "./code.cg2js/cg-tab-control.fasl")
(#x34be94783912e0159d98bc6c2ef7f3cc . "./code.cg2js/cg-template-string.fasl")
(#xff09afabcda52fb966d139c30e4e38a2 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xe0688d1544fd6e3cba4c48f5eb8e2785 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xb595a510b29989a347af418b72f7c637 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x278dd6d57538fd7d427f97d6d3cd5d95 . "./code.cg2js/cg-text-or-combo.fasl")
(#x2f14ba6f10833a060fb4597b090f74bb . "./code.cg2js/cg-text-widget.fasl")
(#xe5e1894df8fea6bfe0089cfe0ecbd226 . "./code.cg2js/cg-timer.fasl")
(#x17d3b2c0a583dec3a5e325b9483adc70 . "./code.cg2js/cg-toggling-widget.fasl")
(#x618fd7b85297134fb5948017101554bb . "./code.cg2js/cg-toolbar.fasl")
(#x98fc802653af5521d3df590f0955dc0d . "./code.cg2js/cg-tooltip.fasl")
(#x0f56399b5d6e22797f9c88454f938e3c . "./code.cg2js/cg-touch.fasl")
(#xf85a692a6e0839410107cb2446501fd4 . "./code.cg2js/cg-trackbar.fasl")
(#xd14c87717017a189c8c4ae280f4faaed . "./code.cg2js/cg-tray.fasl")
(#x6382279ad7dd4d732321e3a9a756339f . "./code.cg2js/cg-up-down-control.fasl")
(#xcdb1fb72ec08ac08da1ef5a8f1dec062 . "./code.cg2js/cg-utility-dialog.fasl")
(#xa22c45250e167d3d260427c3dca4c7dd . "./code.cg2js/cg-web-browser-dde.fasl")
(#xc495311c0b2920e65a55d8f1e542557c . "./code.cg2js/cg-web-browser.fasl")
(#x27346c97b104b9f224707e9e1a3b8eb6 . "./code.cg2js/cg-wrap-string.fasl")
(#xf5bc55cbc3dab1d100d0dac0319efd8e . "./code.cg2js/cg-yes-no-list.fasl")
(#x096f820418666f365cb8912e82a41288 . "./code.cg2js/cg-yes-no-string.fasl")
(#x6a794d30dd989ab3f98688b9a66c16f4 . "./code.cg2js/cg.fasl")
(#xb1fd31e769db1c36af52ea2d1f0cce03 . "./code.idejs/ide-acache.fasl")
(#x4186041e952778d13895e5e06f13f629 . "./code.idejs/ide-apropos.fasl")
(#x6ea6f3066f045198204e1179f7de0daf . "./code.idejs/ide-base.fasl")
(#xcb933eaa82cbe4d41a0d9fc6b00c890f . "./code.idejs/ide-class-browser.fasl")
(#x3a7bdbfde301e1069c7be23faba7d750 . "./code.idejs/ide-class-grapher.fasl")
(#x55b5a1483d8dfcc89b58f99dbf941efb . "./code.idejs/ide-clipboard.fasl")
(#x51a2474ef62f538eff3e15a88ff60656 . "./code.idejs/ide-dev.fasl")
(#x5578c59515a75135d4dd173c61b65368 . "./code.idejs/ide-editor.fasl")
(#x36461fe95e7b3fe8bc4002a38973aed3 . "./code.idejs/ide-find-definitions.fasl")
(#x4e498f9b626a43afceaf73d213539aa1 . "./code.idejs/ide-find-in-files.fasl")
(#x7eb5e3c99fa01779eb40c0af19f3f20f . "./code.idejs/ide-form.fasl")
(#x9a7e63bcfc3673ba5f2f8ee1feeae649 . "./code.idejs/ide-grapher.fasl")
(#x079d1742be676cf17dbea84c2877bc93 . "./code.idejs/ide-inspector.fasl")
(#x4ea82a54c710c09014ada7a93bf9d56c . "./code.idejs/ide-intro.fasl")
(#x0859132602391a8a5bf629500d33a9e0 . "./code.idejs/ide-keyshort.fasl")
(#xa7d588143d2dd65ac5511798d112135e . "./code.idejs/ide-navigator.fasl")
(#x4fdec2f96437021aff8ebe5bdf0b7f93 . "./code.idejs/ide-package-list.fasl")
(#xbf663c7ac18fbad0f433b73dfbd66314 . "./code.idejs/ide-patch-dialog.fasl")
(#xdf90b3430f75a37a4c3fc639078fd348 . "./code.idejs/ide-process-browser.fasl")
(#x815f7d417024135f5c80665558f7f7fa . "./code.idejs/ide-project.fasl")
(#x557d1a3ad3dfba6100eee9d848f6ce2b . "./code.idejs/ide-trace.fasl")
(#x0d40b969ae2d00dc79a32619719bc061 . "./code.idejs/ide-window-list.fasl")
(#x538ac3c2b1cc74e63d5c2bca6a639b70 . "./code.idejs/ide.fasl")
(#xdf73f3983b983ba8030e7ee4702b498a . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
